@@ -19,10 +19,10 @@ set -x
 
 for dir in objects/*/definition.json
 do
-    cat ${dir} | jq . | sponge ${dir}
+    cat ${dir} | jq -S -j . | sponge ${dir}
 done
 
-cat relationships/definition.json | jq . | sponge relationships/definition.json
+cat relationships/definition.json | jq -S -j . | sponge relationships/definition.json
 
 cat schema_objects.json | jq . | sponge schema_objects.json
 cat schema_relationships.json | jq . | sponge schema_relationships.json
